@@ -225,6 +225,7 @@ __all__ = [
     "range",
     "rank",
     "regexp_count",
+    "regexp_instr",
     "regexp_like",
     "regexp_match",
     "regexp_replace",
@@ -816,7 +817,7 @@ def regexp_replace(
 
 
 def regexp_count(
-    string: Expr, pattern: Expr, start: Expr, flags: Expr | None = None
+    string: Expr, pattern: Expr, start: Expr | None = None, flags: Expr | None = None
 ) -> Expr:
     """Returns the number of matches in a string.
 
@@ -825,10 +826,42 @@ def regexp_count(
     """
     if flags is not None:
         flags = flags.expr
-    start = start.expr if start is not None else Expr.expr
+    start = start.expr if start is not None else start
     return Expr(f.regexp_count(string.expr, pattern.expr, start, flags))
 
 
+def regexp_instr(
+    values: Expr,
+    regex: Expr,
+    start: Expr | None = None,
+    n: Expr | None = None,
+    flags: Expr | None = None,
+    sub_expr: Expr | None = None,
+) -> Expr:
+    """Returns the position of a regular expression match in a string.
+
+    Searches ``values`` for the ``n``-th occurrence of ``regex``, starting at position
+    ``start`` (the first position is 1). Returns the starting or ending position based
+    on ``end_position``. Use ``flags`` to control regex behavior and ``sub_expr`` to
+    return the position of a specific capture group instead of the entire match.
+    """
+    start = start.expr if start is not None else None
+    n = n.expr if n is not None else None
+    flags = flags.expr if flags is not None else None
+    sub_expr = sub_expr.expr if sub_expr is not None else None
+
+    return Expr(
+        f.regexp_instr(
+            values.expr,
+            regex.expr,
+            start,
+            n,
+            flags,
+            sub_expr,
+        )
+    )
+
+
 def repeat(string: Expr, n: Expr) -> Expr:
     """Repeats the ``string`` to ``n`` times."""
     return Expr(f.repeat(string.expr, n.expr))
@@ -1779,7 +1812,7 @@ def array_agg(
 
     For example::
 
-        df.select(array_agg(col("a"), order_by="b"))
+        df.aggregate([], array_agg(col("a"), order_by="b"))
     """
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     filter_raw = filter.expr if filter is not None else None
@@ -1941,7 +1974,7 @@ def median(
 
 
 def min(expression: Expr, filter: Expr | None = None) -> Expr:
-    """Returns the minimum value of the argument.
+    """Aggregate function that returns the minimum value of the argument.
 
     If using the builder functions described in ref:`_aggregation` this function ignores
     the options ``order_by``, ``null_treatment``, and ``distinct``.
@@ -2282,7 +2315,7 @@ def first_value(
 
     For example::
 
-        df.select(first_value(col("a"), order_by="ts"))
+        df.aggregate([], first_value(col("a"), order_by="ts"))
     """
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     filter_raw = filter.expr if filter is not None else None
@@ -2319,7 +2352,7 @@ def last_value(
 
     For example::
 
-        df.select(last_value(col("a"), order_by="ts"))
+        df.aggregate([], last_value(col("a"), order_by="ts"))
     """
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     filter_raw = filter.expr if filter is not None else None
@@ -2358,7 +2391,7 @@ def nth_value(
 
     For example::
 
-        df.select(nth_value(col("a"), 2, order_by="ts"))
+        df.aggregate([], nth_value(col("a"), 2, order_by="ts"))
     """
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     filter_raw = filter.expr if filter is not None else None
@@ -2843,7 +2876,7 @@ def string_agg(
 
     For example::
 
-        df.select(string_agg(col("a"), ",", order_by="b"))
+        df.aggregate([], string_agg(col("a"), ",", order_by="b"))
     """
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     filter_raw = filter.expr if filter is not None else None
