@@ -43,7 +43,7 @@ def test_err(df):
     with pytest.raises(Exception) as e_info:
         df["c"]
 
-    for e in ["SchemaError", "FieldNotFound", 'name: "c"']:
+    for e in ["Schema error", "No field named c"]:
         assert e in e_info.value.args[0]
 
     with pytest.raises(Exception) as e_info:
