@@ -29,7 +29,10 @@ from . import generic as helpers
 
 
 def test_no_table(ctx):
-    with pytest.raises(Exception, match="DataFusion error"):
+    with pytest.raises(
+        ValueError,
+        match="^Error during planning: table 'datafusion.public.b' not found$",
+    ):
         ctx.sql("SELECT a FROM b").collect()
 
 
@@ -92,7 +95,7 @@ def test_register_csv(ctx, tmp_path):
     result = pa.Table.from_batches(result)
     assert result.schema == alternative_schema
 
-    with pytest.raises(ValueError, match="Delimiter must be a single character"):
+    with pytest.raises(ValueError, match="delimiter must be a single character"):
         ctx.register_csv("csv4", path, delimiter="wrong")
 
     with pytest.raises(
