@@ -19,7 +19,7 @@ use datafusion::logical_expr::{TypeSignature, Volatility};
 use pyo3::prelude::*;
 
 #[allow(dead_code)]
-#[pyclass(name = "Signature", module = "datafusion.expr", subclass)]
+#[pyclass(frozen, name = "Signature", module = "datafusion.expr", subclass)]
 #[derive(Clone)]
 pub struct PySignature {
     type_signature: TypeSignature,
