@@ -19,17 +19,17 @@ use std::fmt::{self, Display, Formatter};
 
 use datafusion::common::{DataFusionError, ScalarValue};
 use datafusion::logical_expr::{Expr, Window, WindowFrame, WindowFrameBound, WindowFrameUnits};
+use pyo3::IntoPyObjectExt;
 use pyo3::exceptions::PyNotImplementedError;
 use pyo3::prelude::*;
-use pyo3::IntoPyObjectExt;
 
 use super::py_expr_list;
 use crate::common::data_type::PyScalarValue;
 use crate::common::df_schema::PyDFSchema;
-use crate::errors::{py_type_err, PyDataFusionResult};
-use crate::expr::logical_node::LogicalNode;
-use crate::expr::sort_expr::{py_sort_expr_list, PySortExpr};
+use crate::errors::{PyDataFusionResult, py_type_err};
 use crate::expr::PyExpr;
+use crate::expr::logical_node::LogicalNode;
+use crate::expr::sort_expr::{PySortExpr, py_sort_expr_list};
 use crate::sql::logical::PyLogicalPlan;
 
 #[pyclass(frozen, name = "WindowExpr", module = "datafusion.expr", subclass)]
