@@ -23,7 +23,7 @@ use pyo3::prelude::*;
 use pyo3::types::PyBytes;
 
 use crate::context::PySessionContext;
-use crate::errors::{py_datafusion_err, PyDataFusionError, PyDataFusionResult};
+use crate::errors::{PyDataFusionError, PyDataFusionResult, py_datafusion_err, to_datafusion_err};
 use crate::sql::logical::PyLogicalPlan;
 use crate::utils::wait_for_future;
 
@@ -42,6 +42,19 @@ impl PyPlan {
             .map_err(PyDataFusionError::EncodeError)?;
         Ok(PyBytes::new(py, &proto_bytes).into())
     }
+
+    /// Get the JSON representation of the substrait plan
+    fn to_json(&self) -> PyDataFusionResult<String> {
+        let json = serde_json::to_string_pretty(&self.plan).map_err(to_datafusion_err)?;
+        Ok(json)
+    }
+
+    /// Parse a Substrait Plan from its JSON representation
+    #[staticmethod]
+    fn from_json(json: &str) -> PyDataFusionResult<PyPlan> {
+        let plan: Plan = serde_json::from_str(json).map_err(to_datafusion_err)?;
+        Ok(PyPlan { plan })
+    }
 }
 
 impl From<PyPlan> for Plan {
